// **************************************************************************
//
//  File:  I2C.c				Version: 0.51
//
//  Date:  October 20, 2004		
//
//  P1 CM64 Communication code and I2C implementation
//
//  Copyright 2004-2006, SOC Robotics, Inc.
//
// **************************************************************************
/*
 * Copyright (C) 2005-2006 by SOC Robotics, Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Machines, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OC MACHINES, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-machines.com
 */

/*
 * $Log: P1_I2C.c,v $
 * Revision 0.54  2005/06/04 18:36:42  steve
 * Preview release
 *
 */
 #include "P1_Main.h"
#include "ccblkfn.h"
#include "sysreg.h"
#include "P1_I2C.h"
#include "SAA7113.h"
#include "ICM109S.h" 
#include "P1_Extern.h"

//****************************************************************
//****************************************************************
// Start Code

void Set_SDA(char mode)
{
	// Set SDA high by making it an input
	if(mode==HIGH) {
		*pFIO_DIR |= 0x0001;
		*pFIO_FLAG_S = 0x0001;
		//*pFIO_DIR &= 0xFFFE;
		//*pFIO_INEN |= 0x0001;
		//*pFIO_FLAG_S = 0x0001;
		}
	// Make SDA low by making it an output
	else  {
		*pFIO_DIR |= 0x0001;
		*pFIO_FLAG_C = 0x0001;
		}
}

void Set_SCL(char mode)
{
	if(mode==HIGH) *pFIO_FLAG_S = 0x0002;
	else   *pFIO_FLAG_C = 0x0002;
}

//  Set PF0 and PF1 to outputs
void Init_I2C(void)
{
	*pFIO_DIR |= 0x0002;
	Set_SCL(HIGH);
	Set_SDA(HIGH);  
}

void Pulse_SCL(void)
{
	Delay(I2C_PERIOD);
	
	//  Set SCL high
	Set_SCL(HIGH);
	
	//  Delay
	Delay(I2C_PERIOD);
	
	// Set SCL Low
	Set_SCL(LOW);
	
	Delay(I2C_PERIOD);
}

void I2C_Start(void)
{
	Set_SDA(LOW);
	Delay(I2C_PERIOD*2);
	Set_SCL(LOW);
	Delay(I2C_PERIOD*2);
}

void I2C_Stop(void)
{
	Set_SCL(HIGH);
	Delay(I2C_PERIOD*2);
	Set_SDA(HIGH);
	Delay(I2C_PERIOD*2);
}

void I2C_RepeatedStart(void)
{
	Set_SDA(HIGH);
//	Delay(I2C_PERIOD);
	Set_SCL(HIGH);
	Delay(I2C_PERIOD);
	I2C_Start();
}


//  Send the SAA7113 Write or Read address command
int Write_I2C_Control(unsigned char address)
{
	int i,ACKvalue;
	
	//  Send the device Address command MSB first
	for(i=0; i<8; i++) {
		if(address&0x80) Set_SDA(HIGH);
		else Set_SDA(LOW);
		Pulse_SCL();
		address <<= 1;
		}
	//Set_SDA(HIGH);
	*pFIO_INEN |= 0x0001;
	*pFIO_DIR &= 0xFFFE;
	
	// Wait for Ack - correct ack is active low
	Delay(I2C_PERIOD);
	Set_SCL(HIGH);
	Delay(I2C_PERIOD);
	ACKvalue = *pFIO_FLAG_D;  // read PF0
	Delay(I2C_PERIOD);
	Set_SCL(LOW);
	Delay(I2C_PERIOD);

	// Make SDA output
	*pFIO_INEN &= 0xFFFE;
	Set_SDA(LOW);
	Delay(I2C_PERIOD);
	
	//  Return state of acknowledgement
	if(!(ACKvalue&0x0001))  return 1;
	else return -1;
}

int Write_I2C_Byte(unsigned char subaddress)
{
	int i,ACKvalue;
	
	//  Send the device Address command MSB first
	for(i=0; i<8; i++) {
		if(subaddress&0x80) Set_SDA(HIGH);
		else Set_SDA(LOW);
		Pulse_SCL();
		subaddress <<= 1;
		}
	//Set_SDA(HIGH);
	*pFIO_INEN |= 0x0001;
	*pFIO_DIR &= 0xFFFE;
	
	// Wait for Ack
	Delay(I2C_PERIOD);
	Set_SCL(HIGH);
	Delay(I2C_PERIOD);
	ACKvalue = *pFIO_FLAG_D;  // read PF0
	Delay(I2C_PERIOD);
	Set_SCL(LOW);
	Delay(I2C_PERIOD);
	
	// Make SDA output
	*pFIO_INEN &= 0xFFFE;
	Set_SDA(LOW);
	Delay(I2C_PERIOD);
	
	//  Return state of acknowledgement
	if(ACKvalue&0x0001)  return 1;
	else return -1;
}

unsigned char Read_I2C_Byte()
{
	unsigned char inbyte;
	int i,ACKvalue;
	
	//  Make PF0 (SDA) input
	//Set_SDA(HIGH);
	*pFIO_INEN |= 0x0001;
	*pFIO_DIR &= 0xFFFE;
	
	//  Read a byte from SAA7113
	inbyte = 0x00;
	Delay(I2C_PERIOD);
	for(i=0; i<8; i++) {
		Set_SCL(HIGH);
		Delay(I2C_PERIOD/2);
		if(*pFIO_FLAG_D&0x0001) inbyte |= 0x01;
		Delay(I2C_PERIOD/2);
		Set_SCL(LOW);
		if(i<7) inbyte <<= 1;
		Delay(I2C_PERIOD);
		}
	
	// Send back ACK
	//    Make SDA an output
	Delay(I2C_PERIOD);	
	*pFIO_INEN &= 0xFFFE;
	Set_SDA(HIGH);  // Active low -> good ack
	Delay(I2C_PERIOD);
	Set_SCL(HIGH);
	Delay(I2C_PERIOD);
//	ACKvalue = *pFIO_FLAG_D;  // read PF0
//	Delay(I2C_PERIOD);
	Set_SCL(LOW);
	Delay(I2C_PERIOD);
	Set_SDA(LOW);
	Delay(I2C_PERIOD);
			
	//  Return read byte
	return inbyte;
}

// Write data byte to CM64 SAA7113 video decoder chip using I2C
//  If I2C device acknowledges transfers return 1 else return -1
int Write_CM64(unsigned char Sub_ADD, unsigned char Data)
{
	I2C_Start();			   			// Set I2C start condition

 	Write_I2C_Control(SAA7113_Write_ADD);	// Send the SAA7113 control Byte

 	Write_I2C_Byte(Sub_ADD);			// Send the SAA7113 sub Address
  
 	Write_I2C_Byte(Data);				// Send the SAA7113 Data
 
 	I2C_Stop();	  					    // Set I2C Stop condition
 	
 	return 1;
}


//  Read a data byte from the CM64 SAA7113 chip using I2C
unsigned char Read_CM64(unsigned char Sub_ADD)
{
 	unsigned char inByte;			   		// Temp RAM for SAA7113 Read
 
 	I2C_Start();			   			// Set I2C start condition

 	Write_I2C_Control(SAA7113_Write_ADD);	// Send the SAA7113 control Byte
 									    // Dummy write to set address
 									    
 	Write_I2C_Byte(Sub_ADD);			// Send the SAA7113 internal Address
 
 	I2C_RepeatedStart();			   	// Set I2C Repeat start condition
 
 	Write_I2C_Control(SAA7113_Read_ADD);	// Send the SAA7113 control Byte
 
 	inByte = Read_I2C_Byte();			// Read data from SAA7113
 
 	I2C_Stop();	  					    // Set I2C Stop condition

 	return inByte;						// Return data from SAA7113

}


//  Configure SAA7113 by writing setup bytes using I2C interface
//
//   - Analog input AI11 CVBS format, analog anti-alias filter and AGC active
//   - Automatic field detection enabled, PAL BDGHI or NTSC M standard expected
//   - Standard ITU 656 output format enabled, VBI-data slicer disabled
//   - Contrast, brightness and saturation control in accordance with ITU standards
//   - Chrominance processing with nominal bandwidth
void Init_SAA7113(short videochannel)
{
	Write_CM64(CM64_INCREMENTDELAY,0x08);
	
	if(videochannel==CM64_VIDEOCHANNEL2)  Write_CM64(CM64_ANINPUTCNTL1,0xc1);  // Video Channel 12
	else  Write_CM64(CM64_ANINPUTCNTL1,0xc0);  //  Video channel 11
	
	Write_CM64(CM64_ANINPUTCNTL2,0x33);
	Write_CM64(CM64_ANINPUTCNTL3,0x00);
	Write_CM64(CM64_ANINPUTCNTL4,0x00);
	Write_CM64(CM64_HSYNCSTART,0xe9);
	Write_CM64(CM64_HSYNCSTOP,0x0d);
	
	//  Note: If TV signal poor set to 0x90 rather than 0x98
	Write_CM64(CM64_SYNCCNTL,0x90);
	Write_CM64(CM64_LUMINANCECNTL,0x01);
	Write_CM64(CM64_LUMINANCEBRIGHTNESS,0x80);
	Write_CM64(CM64_LUMINANCECONTRAST,0x47);
	Write_CM64(CM64_CHROMASATURATION,0x40);
	Write_CM64(CM64_CHROMAHUECNTL,0x00);
	Write_CM64(CM64_CHROMACNTL,0x01);
	Write_CM64(CM64_CHROMAGAINCNTL,0x2a);
	
	//  Note:  If D3 = 1 then 242 lines per field else 240 lines per field
	Write_CM64(CM64_FORMATDELAYCNTL,0x08);
	
	//  Note:  If D0 = 1 color on else off, enable YUV output and RTSx lines
	Write_CM64(CM64_OUTCNTL1,0x0D);
	
	Write_CM64(CM64_OUTCNTL2,0x01);
	Write_CM64(CM64_OUTCNTL3,0x00);
	Write_CM64(CM64_VGATE1START,0x00);
	Write_CM64(CM64_VGATE1STOP,0x00);
	Write_CM64(CM64_VGATE1MSB,0x00);
	Write_CM64(CM64_AC1,0x02);
	Write_CM64(CM64_LCR2,0xff);
	Write_CM64(CM64_LCR3,0xff);
	Write_CM64(CM64_LCR4,0xff);
	Write_CM64(CM64_LCR5,0xff);
	Write_CM64(CM64_LCR6,0xff);
	Write_CM64(CM64_LCR7,0xff);
	Write_CM64(CM64_LCR8,0xff);
	Write_CM64(CM64_LCR9,0xff);
	Write_CM64(CM64_LCR10,0xff);
	Write_CM64(CM64_LCR11,0xff);
	Write_CM64(CM64_LCR12,0xff);
	Write_CM64(CM64_LCR13,0xff);
	Write_CM64(CM64_LCR14,0xff);
	Write_CM64(CM64_LCR15,0xff);
	Write_CM64(CM64_LCR16,0xff);
	Write_CM64(CM64_LCR17,0xff);
	Write_CM64(CM64_LCR18,0xff);
	Write_CM64(CM64_LCR19,0xff);
	Write_CM64(CM64_LCR20,0xff);
	Write_CM64(CM64_LCR21,0xff);
	Write_CM64(CM64_LCR22,0xff);
	Write_CM64(CM64_LCR23,0xff);
	Write_CM64(CM64_LCR24,0xff);
	Write_CM64(CM64_FC,0x00);
	Write_CM64(CM64_HOFF,0x54);
	Write_CM64(CM64_VOFF,0x07);
	Write_CM64(CM64_HVOFF,0x83);
if(0) {	}
	
}

//  Reset CM64 SAA7113 
void Reset_SAA7113()
{
	//  Reset SAA7113 - PF4 active low - hold for 100usec, 
	//     release and recover for 10msec
	*pFIO_DIR |= 0x0010;  // Make PF4 output
	*pFIO_FLAG_D |= 0x0010;
	
	// Reset SAA7113
	*pFIO_FLAG_C = 0x0010;  
	Delay(100);    // wait 100usec	
//	Init_I2C();
	*pFIO_FLAG_S = 0x0010;  // Release reset
	Delay(3000);  // wait 10msec
}

//  See if CM64 is sync'd to video signal
short  CM64_VideoStatus()
{
	unsigned char inchar;
	
	inchar = Read_CM64(CM64_STATUSBYTE);
	if(inchar&0x40) return 0;
	else return 1;
}

//  Gracefully shut down video DMA and PPI operation
void Halt_VideoDMAPPI()
{	
	// Turn DMA flow off - DMA completes current operation and stops
	*pDMA0_CONFIG &= 0x0FFF;
	Delay(50000);
	
	// Turn the PPI off
	*pPPI_CONTROL &= 0xFFFE;
}


// ***********************************************************************
//
//  CM64 Initialization and Setup
//
//  The CM64 is a analog video decoder module that attaches to the P1
//  video input port connector J3.
//
//  This routine sets up the PPI/PF port, programs the SAA7113 video 
//  decoder via software controlled I2C port and sets up the DMA transfer using
//  a ping/pong buffer architecture.  The SAA7113 is setup to operate in 
//  ITU 656 mode - the Blackfin is configured to receive in ITU 656 mode.
//
//  In ITU 656 mode Vsync, Field and Hsync are not used
//
//  **********************************************************************
void Init_CM64(short videochannel,unsigned char skipmode)
{
	short notsynced;
	long videocnt=0;
	
	//  Setup PPI/PF port assignments
	//
	//     - PF0 - SDA (I2C)  Blackfin output
	//     - PF1 - SCL (I2C)  Blackfin output
	//     - PF4 - Reset Blackfin output - active low
	//     - PF2 - FID RTC0 - let this pin float immediately after reset release
	//     - PPI_FS2 - VS RTS0 - I/O - slave address - internal pull up - VS
	//           On power up if low - sets slave address to 48H write/49H read
	//                       if high - sets address to 4AH write/4BH read
	//     - PPI_FS1 - HS RTS1 - I/O

	// Enable the DMA transfer registers for Channel 0
	//   Maybe this should be after Init_PPI
  	Init_DMA_CH0(skipmode);
	
	//  Initialize the I2C - PF0 is SDA, PF1 is SCL
	Init_I2C();
	
	//  Reset SAA7113 - PF4 active low - hold for 100usec, 
	//     release and recover for 4msec
	Reset_SAA7113();

	//  Write initialization parameters to SAA7113 - select video channel 1 or 2
	Init_SAA7113(videochannel);
	
	//  Delay for enough time for SAA7113 to sync to video
	//   TODO
	Delay(1);
	
	notsynced = 1;
	//  See if SAA7113 is sync'd to video signal (ie check status)
	while(notsynced)  {
		notsynced = CM64_VideoStatus();
		Delay(10);
		if(videocnt++>1000)  {
			puts("No video signal on CM64\n\r");
			notsynced = 0;
			}
		}
		

	//  Init PPI
	Init_PPI_CM64(skipmode);
		
}

//  Power up or down CM200
void CM200_Power(unsigned char mode)
{
	//  PF5 (pin 25) controls power to ICM109S 
	//     if 1 sensor is on
	*pFIO_DIR |= 0x0020;  // Make PF5 output
	*pFIO_FLAG_D |= 0x0020;
	
	// Set PF5 to 0
	if(mode==ON)  *pFIO_FLAG_S = 0x0020;
	else  *pFIO_FLAG_C = 0x0020;
}

//  Reset CM200 ICM109S, set SIF-ID (PF2) low and 
//    enable outputs OE (PF6)
void Reset_CM200()
{
	//  Reset ICM109S - PF4 active low - hold for 100usec, 
	//     release and recover for 0.3msec
	*pFIO_DIR |= 0x0054;  // Make PF6,4,2 outputs
	*pFIO_FLAG_D |= 0x0054;
	
	// Set SIF_ID to 0
	*pFIO_FLAG_C = 0x0004; 
	
	// Reset ICM109S
	*pFIO_FLAG_C = 0x0010;  
	Delay(100);    // wait 100usec	

	*pFIO_FLAG_S = 0x0010;  // Release reset
	Delay(300);  // wait 0.3msec

	// Set OE low - enable output buffers
	*pFIO_FLAG_C = 0x0040; 
}

// Write data byte to CM200 ICM109S CMOS imaging chip using I2C
//  If I2C device acknowledges transfers return 1 else return -1
int Write_CM200(unsigned char Sub_ADD, unsigned char Data)
{
	I2C_Start();			   			// Set I2C start condition

 	Write_I2C_Control(ICM109S_Write_ADD);	// Send the ICM109 control Byte

 	Write_I2C_Byte(Sub_ADD);			// Send the ICM109 sub Address
  
 	Write_I2C_Byte(Data);				// Send the ICM109 Data
 
 	I2C_Stop();	  					    // Set I2C Stop condition
 	
 	return 1;
}


//  Read a data byte from the CM200 ICM109S chip using I2C
unsigned char Read_CM200(unsigned char Sub_ADD)
{
 	unsigned char inByte;
 
 	I2C_Start();			   			// Set I2C start condition

 	Write_I2C_Control(ICM109S_Write_ADD);	// Send the ICM109 control Byte
 									    // Dummy write to set address
 									    
 	Write_I2C_Byte(Sub_ADD);			// Send the ICM109 internal Address
 
 	I2C_RepeatedStart();			   	// Set I2C Repeat start condition
 
 	Write_I2C_Control(ICM109S_Read_ADD);	// Send the ICM109 control Byte
 
 	inByte = Read_I2C_Byte();			// Read data from ICM109
 
 	I2C_Stop();	  					    // Set I2C Stop condition

 	return inByte;						// Return data from ICM109

}

//  **************************************************************************
//  Init ICM109S
//
//  Configure ICM109S by writing setup bytes using I2C interface
//  The ICM109S is a 2MPixel CMOS sensor with a 1600x1200x10bit imaging array
//  The sensor operates at upto 20fps (slower rates possible) and programmable 
//  window size and start address.
void Init_ICM109S()
{
	short i;
	// Dead pixel array init
	//     - first char=1 if pixel dead
	//     - second char is I2C address base
	//     - next four chars is col(l,h) and row(l,h) address
	unsigned char deadpixel[24][6] = {
		0, 0xC0, 0,0,0,0,
		0, 0xC4, 0,0,0,0,
		0, 0xC8, 0,0,0,0,
		0, 0xCC, 0,0,0,0,
		0, 0xD0, 0,0,0,0,
		0, 0xD4, 0,0,0,0,
		0, 0xD8, 0,0,0,0,
		0, 0xDC, 0,0,0,0,
		0, 0xE0, 0,0,0,0,
		0, 0xE4, 0,0,0,0,
		0, 0xE8, 0,0,0,0,
		0, 0xEC, 0,0,0,0,
		0, 0xF0, 0,0,0,0,
		0, 0xF4, 0,0,0,0,
		0, 0xF8, 0,0,0,0,
		0, 0xFC, 0,0,0,0,
		0, 0x60, 0,0,0,0,
		0, 0x64, 0,0,0,0,
		0, 0x68, 0,0,0,0,
		0, 0x6C, 0,0,0,0,
		0, 0x70, 0,0,0,0,
		0, 0x74, 0,0,0,0,
		0, 0x7C, 0,0,0,0,
		0, 0x74, 0,0,0,0,
		};

	// Initialize ICM109S - 2 megapixel sensor	
	
	// Main control byte:
	//    D0 - 0 normal, 1 single frame
	//    D2 - 1 load new exposure time - see EXPOSETIME
	//    D6:4 - 0 20fps, 1 10fps, 2 7fps, 3 3.5fps, 4 3fps, 5 2fps, 6 1.5fps, 7 .7
	//    D7 - activate latent change registers
	Write_CM200(CM200_PARTCONTROL,0x00);
	
	// Set Vsync and Hsync parameters
	//   D0 - wave table enable
	//   D1 - Hsync polarity - 1 active high
	//   D2 - Ysync polarity - 1 active high
	//   D3 - auto dark correction enabled
	//   D4 - 0 wave table, 1 default
	//   D5 - Flash polarity, 1 active high
	//   D6 - Blank polarity, 1 active high
	Write_CM200(CM200_TIMINGCONTROL_LOW,0x11);
	//   D3 - trigger single frame capture
	Write_CM200(CM200_TIMINGCONTROL_HIGH,0x00);
	
	// Frame size and start location
	Write_CM200(CM200_WIDTH_L,0xDC);
	Write_CM200(CM200_WIDTH_H,0x05);
	Write_CM200(CM200_HEIGHT_L,0x4C);
	Write_CM200(CM200_HEIGHT_H,0x04);
	Write_CM200(CM200_COLBEGIN_L,0x64);
	Write_CM200(CM200_COLBEFIN_H,0x00);
	Write_CM200(CM200_ROWBEGIN_L,0x0A);
	Write_CM200(CM200_ROWBEGIN_H,0x00);
	
	// Note Hsync and Vsync may be related to frame size settings
	Write_CM200(CM200_HSYNCEND_L,0x40);
	Write_CM200(CM200_HSYNCEND_H,0x00);
	Write_CM200(CM200_VSYNCEND_L,0x03);
	Write_CM200(CM200_VSYNCEND_H,0x00);
	
	// Default exposure time is 1299 lines
	Write_CM200(CM200_EXPOSETIME_L,0x13);
	Write_CM200(CM200_EXPOSETIME_H,0x05);
	
	// Gain settings
	Write_CM200(CM200_M1_L,0x00);
	Write_CM200(CM200_M1_H,0x01);
	Write_CM200(CM200_M2_L,0x00);
	Write_CM200(CM200_M2_H,0x01);
	Write_CM200(CM200_M3_L,0x00);
	Write_CM200(CM200_M3_H,0x01);
	Write_CM200(CM200_M4_L,0x00);
	Write_CM200(CM200_M4_H,0x01);
	Write_CM200(CM200_DARKDATA_L,0x00);
	Write_CM200(CM200_DARKDATA_H,0x00);
	
	// Dead pixel level enable settings
	Write_CM200(CM200_HIGHLIMIT_L,0xFF);
	Write_CM200(CM200_HIGHLIMIT_H,0x03);
	Write_CM200(CM200_LOWLIMIT_L,0x00);
	Write_CM200(CM200_LOWLIMIT_H,0x00);
	
	// Output format = 0 10bit raw
	Write_CM200(CM200_INOUTSEL,0x00);
	
	// Reserver settings
	//Write_CM200(CM200_RAMPSEL,0x00);
	//Write_CM200(CM200_DSRST_L,0x00);
	//Write_CM200(CM200_DSRST_H,0x00);
	//Write_CM200(CM200_DSDATA_L,0xD0);
	//Write_CM200(CM200_DSDATA_L,0x07);
	
	// Sub id read from D4:0 during reset
	Write_CM200(CM200_ID_L,0x90);
	Write_CM200(CM200_ID_H,0xD0);
	
	// Flash light control - not applicable to ICM109S
	//Write_CM200(CM200_FLASHBEGIN_L,0x0A);
	//Write_CM200(CM200_FLASHBEGIN_H,0x05);
	//Write_CM200(CM200_FLASHEND_L,0x1E);
	//Write_CM200(CM200_FLASHEND_H,0x05);
	
	// Blank begin/end location
	Write_CM200(CM200_BWIDTHBEGIN_L,0x68);
	Write_CM200(CM200_BWIDTHBEGIN_H,0x00);
	Write_CM200(CM200_BWIDTHEND_L,0xA7);
	Write_CM200(CM200_BWIDTHEND_H,0x06);
	Write_CM200(CM200_BHEIGHTBEGIN_L,0x0E);
	Write_CM200(CM200_BHEIGHTBEGIN_H,0x00);
	Write_CM200(CM200_BHEIGHTEND_L,0xBD);
	Write_CM200(CM200_BHEIGHTEND_H,0x04);
	Write_CM200(CM200_RSTSEL,0x40);
	
	// Reserved settings
	//Write_CM200(CM200_BITCONTROL,0xC0);
	//Write_CM200(CM200_WTBEGIN_L,0x00);
	//Write_CM200(CM200_WTBEGIN_H,0x00);
	//Write_CM200(CM200_WTEND_L,0xF8);
	//Write_CM200(CM200_WTEND_H,0x07);
	//Write_CM200(CM200_SUBENTIME_L,0xEA);
	//Write_CM200(CM200_SUBENTIME_H,0x06);
	
	// Current frame settings - ?
	Write_CM200(CM200_WIDTH_LC,0x08);
	Write_CM200(CM200_WIDTH_HC,0x07);
	Write_CM200(CM200_HEIGHT_LC,0x14);
	Write_CM200(CM200_HEIGHT_HC,0x05);
	Write_CM200(CM200_COLBEGIN_LC,0x64);
	Write_CM200(CM200_COLBEGIN_HC,0x00);
	Write_CM200(CM200_ROWBEGIN_LC,0x0A);
	Write_CM200(CM200_ROWBEGIN_HC,0x00);
	Write_CM200(CM200_HSYNCEND_LC,0x40);
	Write_CM200(CM200_HSYNCEND_HC,0x00);
	Write_CM200(CM200_VSYNCEND_LC,0x03);
	Write_CM200(CM200_VSYNCEND_HC,0x00);
	Write_CM200(CM200_PARTCONTROL_C,0x00);
	Write_CM200(CM200_WTBEGIN_LC,0x00);
	Write_CM200(CM200_WTBEGIN_HC,0x00);
	Write_CM200(CM200_WTEND_LC,0xF8);
	Write_CM200(CM200_WTEND_HC,0x07);
	
	// PLL clock setting - 6MHz clock
	//    00000 - x1
	//    00001 - x2
	//    00011 - x4
	//    00111 - x8
	//    01111 0 x16
	Write_CM200(CM200_PLL,0x07);
	
	//   Write dead pixel array
	for(i=0; i<24; i++) {
		if(deadpixel[i][0]==1) {
			Write_CM200(deadpixel[i][1],deadpixel[i][2]);
			Write_CM200(deadpixel[i][1]+0x01,deadpixel[i][3]);
			Write_CM200(deadpixel[i][1]+0x02,deadpixel[i][4]);
			Write_CM200(deadpixel[i][1]+0x03,deadpixel[i][5]);
			}
		}
	
}

// ***********************************************************************
//
//  CM200 Initialization and Setup
//
//  The CM200 is a megapixel digital video decoder module that attaches to the P1
//  video input port connector J3.
//
//  This routine sets up the PPI/PF port, programs the ICM109S video 
//  decoder via a software controlled I2C port and sets up the DMA transfer using
//  a ping/pong buffer architecture.  The ICM109s outputs 11bit video so the 
//  Blackfin PPI port is setup to read 11bit data and Vsync, Hsync.
//
//  **********************************************************************
void Init_CM200()
{
	//  Setup PPI/PF port assignments
	//
	//     - PF0 - SDA (I2C)  Blackfin output
	//     - PF1 - SCL (I2C)  Blackfin output
	//     - PF4 - Reset Blackfin output - active low
	//     - PF2 - SIF-ID - set this pin low to select Write I2C address 0x20
	//     - PPI_FS2 - VSYNC
	//     - PPI_FS1 - HSYNC

	// Enable the DMA transfer registers for Channel 0
	//   Maybe this should be after Init_PPI
  	Init_DMA_CH0_CM200();
	
  	
	//  Initialize the I2C - PF0 is SDA, PF1 is SCL
	Init_I2C();

	// Enable power to ICM109
	CM200_Power(ON);
	
	//  Reset ICM109S - PF4 active low - hold for 100usec, 
	//     release and recover for ?msec
	Reset_CM200();

	//  Write initialization parameters to ICM109S
	Init_ICM109S();
	
	//  Delay for enough time for ICM109 to sync
	Delay(1);
	
	//  Init PPI
	Init_PPI_CM200();
		
}

