// **************************************************************************
//
//  File:  P1_SPI_Flash.c				Version: 0.67
//
//  Date:  October 20, 2004		
//
//  Utilities for read/write Serial SPI Flash for P1 Blackfin Monitor
//
// **************************************************************************
/*
 * Copyright (C) 2005-2006 by SOC Robotics, Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Machines, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OC MACHINES, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-machines.com
 */

/*
 * $Log: P1_Blackfin_Monitor.c,v $
 * Revision 0.54  2005/06/04 18:36:42  steve
 * Preview release
 *
 */
#include "ccblkfn.h"
#include "sysreg.h"
#include "math.h"

#include "P1_Main.h"
#include "P1_I2C.h"
#include "SAA7113.h"  
#include "P1_Extern.h"

//****************************************************************
//****************************************************************


void FlashChipSelect(int FlashChip,int mode)
{
	Delay(1);
	if((FlashChip==1)&&(mode==LOW)) *pSPI_FLG = 0xFB04;
	else if((FlashChip==1)&&(mode==HIGH)) *pSPI_FLG = 0xFF04;
	else if(FlashChip==2&&mode==LOW) *pSPI_FLG &= 0xF7FF;
	else if(FlashChip==2&&mode==HIGH) *pSPI_FLG |= 0x0800;
	Delay(1);
}

// TODO - break if no reception
void SPI_MasterTransmit(unsigned char cData)
{
	unsigned int inword,outword;
    long cnt;
	
    //  Enable transmit operation
    *pSPI_CTL = 0x5C0D;
	cnt = 0;

	// Wait until TX buffer empty - full=1
	if(SPI_State==0x01) 
	    while(*pSPI_STAT & 0x0004) ;
	

    // Start transmisstion
	*pSPI_TDBR = cData;
		
	// Do dummy read to transfer data
//	inword = *pSPI_RDBR;
	
	//  See if error bits are set - clear if set
	inword = *pSPI_STAT;
	if(inword&0x0014) *pSPI_STAT |= 0x0014;
	
	// Wait for transmission to complete
//	puts("wait for spi tx to complete\n\r");
	while(!(*pSPI_STAT & 0x0001)) ;
	
	SPI_State = 0x01;
}

// TODO - break if no reception
unsigned char SPI_MasterReceive(void)
{
	unsigned int inword;
    // Wait for reception to complete
    // Is a dummy read required?
    *pSPI_CTL = 0x5C0C;
    
//	*pSPI_TDBR = 0x00;
	SPI_State = 0x02;

	inword = *pSPI_RDBR;
	
	while(!(*pSPI_STAT & 0x0020)) ;
	
	return (*pSPI_RDBR & 0x00ff);
}



//  Read address consists of 4 zero bits+11 page bits and 9 byte start
//    AT45DB041 has 264 bytes per page - 2048 pages
//    AT45DB081 has 264 bytes per page - 
//    AT45DB161 has 512 bytes per page
//    264 does not divide into 2^16 evenly
void ConvertFlashAddress(long address)
{
 	unsigned int pages,bytestart,superpages;
 	long superaddress;
	
 	//superaddress = extendedaddress*65536+address;
 	if(address < f1bytesize) {
	    pages = address/264;
	    superpages = pages/256;
	    bytestart = address%264;
        FlashAddress[0] = ((superpages&0x00ff)<<1) + ((pages&0x00ff)>>7);
	    FlashAddress[1] = ((pages&0x00ff)<<1) + ((bytestart>>8)&0x00FF);
	    FlashAddress[2] = bytestart & 0x00FF;
 		}
 	else {
 		FlashAddress[0] = FlashAddress[1] = FlashAddress[2] = 0;
 		}
}

//  Flash continuous read command for AT45DB041B
//  The AT45DB041B has 2048 264byte pages
void SPI_StartReadFlashPage(long address)
{
 	 int i;
	 
	 // Send Continuous read command
 	 SPI_MasterTransmit(0x68);
//	 puts("Send read command\n\r");
	 
	 // Convert address to 24 bit address
	 ConvertFlashAddress(address);
	 for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
//	 puts("Address\n\r");
	 
	 //  32 dummy bits
	 for(i=0; i<4; i++)  SPI_MasterTransmit(0x00);
//	 puts("Send dummy bits\n\r");
}

//  Flash Buffer Read command for AT45DB041B
void SPI_ReadFlashBuffer(long address,int buffernumber)
{
 	 int i;
	 
	 // Send buffer read command
	 if(buffernumber==1) SPI_MasterTransmit(0x54);
	 else SPI_MasterTransmit(0x56);

//	 puts("Send read command\n\r");
	 
	 // Convert address to 24 bit address
	 ConvertFlashAddress(address);
	 FlashAddress[0] = 0x00;
	 for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
//	 puts("Address\n\r");
	 
	 //  8 dummy bits
	 SPI_MasterTransmit(0x00);
//	 puts("Send dummy bits\n\r");
}

void DisplayExtendedAddress(unsigned int extendedaddress)
{
	unsigned char crcvalue,tempvalue;
	
	crcvalue = 0x00;
	puts(":02000004");
	crcvalue += 0x02;
	crcvalue += 0x04;
	addresstohex(extendedaddress);
	puts(hexaddress);
	tempvalue = (extendedaddress&0xff00)>>8;
	crcvalue += tempvalue;
	tempvalue = extendedaddress&0x00ff;
	crcvalue += tempvalue;
	bytetohex((~crcvalue)+0x01);
	puts(hexbyte);
	puts("\n\r");
}

//  Display P1 Blackfin Flash memory
//
//    There are two Serial Flash devices selected by PF2 and PF3
//
//    If extendedaddress nonzero or numbyte > 65536 then output 
//    Extended Address Record at the appropriate time
//
//   Note:  The Atmel Flash must be read continuously or incorrect data is 
//          returned.
void DisplaySerialFlash(long startaddress,long numbytes,char readmode)
{
	 unsigned char firstrecord,databytes[INTELHEXBYTESPERLINE];
 	 int i;
	 long recordaddress,endaddress,address,extendedaddress,oldextendedaddress;

	 //  Set PF2 low - Flash CS
//	 puts("CS Low\n\r");
	 FlashChipSelect(1,LOW);

//	 if(readmode==READCONTINUOUS)  SPI_StartReadFlashPage(address);
//	 else if(readmode==READBUFFER) SPI_ReadFlashBuffer(address,1);
	 if(readmode==READCONTINUOUS)  SPI_StartReadFlashPage(startaddress);
	 else if(readmode==READBUFFER) SPI_ReadFlashBuffer(startaddress,1);
	 
     address = startaddress;
     endaddress = startaddress + numbytes;
     oldextendedaddress = address/65536;

	 bytesonline = INTELHEXBYTESPERLINE;
	 firstrecord = YES;

	 while(address<endaddress) {
	 	 extendedaddress = address/65536;
	     recordaddress = address - extendedaddress*65536; 
		 
		 // Output extended record if start address > 65536
		 if((firstrecord==YES)&&(extendedaddress>0))  {
			 //puts("-1-");
			 firstrecord = NO;
		     DisplayExtendedAddress(extendedaddress);
			 }	
			 
		 // Traversed 65536 boundary?			 
	     else if(extendedaddress>oldextendedaddress) {
			 //puts("-2-");
		     oldextendedaddress = extendedaddress;
			 DisplayExtendedAddress(extendedaddress);
		 	 }
		
		 // Generate current Hex record
		 else {
		     firstrecord = NO;
		     if((recordaddress+bytesonline)>65536) {
				 //puts("-3-");
			     bytesonline = (65536 - recordaddress);
			     
	     		 for(i=0; i<bytesonline; i++) databytes[i] = SPI_MasterReceive();
	     		 
			     DisplayIntelRecord(recordaddress,bytesonline,databytes);
			     address += bytesonline;
				 bytesonline = INTELHEXBYTESPERLINE;
				 }
			 else if((recordaddress+(long)bytesonline + extendedaddress*65536)>endaddress) {
				 //puts("-3-3-");
			     bytesonline = endaddress - (recordaddress + extendedaddress*65536);

	     		 for(i=0; i<bytesonline; i++) databytes[i] = SPI_MasterReceive();
	         
			     DisplayIntelRecord(recordaddress,bytesonline,databytes);
			     address += bytesonline;
				 bytesonline = INTELHEXBYTESPERLINE;
				 }
			 else {  
	     		 for(i=0; i<bytesonline; i++) databytes[i] = SPI_MasterReceive();
			     DisplayIntelRecord(recordaddress,bytesonline,databytes); 
			     address += bytesonline; 
				 }
			 }
		}			 
		 
	 // Set PC4 high - deselect Flash chip
	 FlashChipSelect(1,HIGH);  
	 puts(":00000001FF\n\r");
}


//  Read the status of Flash chip 1 or 2
unsigned char FlashStatus(int FlashChip)
{
	unsigned char flashstatus;
	
	FlashChipSelect(FlashChip,LOW);
	
	//  Flash Status
	SPI_MasterTransmit(0x57);
	flashstatus = SPI_MasterReceive();
	
	FlashChipSelect(FlashChip,HIGH);
	
	return flashstatus;
}

//  Transfer contents of a Flash page to buffer 1 or 2
void FlashPageToBuffer(int FlashChip,int page, int bufferno)
{
	int i;
	
	FlashChipSelect(FlashChip,LOW);
	
	if(bufferno==1) SPI_MasterTransmit(0x53);
	else SPI_MasterTransmit(0x55);

	// Convert address to 24 bit address
	ConvertFlashAddress(page*264);
	for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);		
	FlashChipSelect(FlashChip,HIGH);	

	//  Wait until erase finishes
	while(!(FlashStatus(1)&0x80))  ;
}

// Erase a Page and wait until done
void FlashPageErase(int FlashChip,int page)
{
	int i;
	
	FlashChipSelect(FlashChip,LOW);
	
	//  Erase a Flash page
	SPI_MasterTransmit(0x81);
	 
	// Convert address to 24 bit address
	ConvertFlashAddress(page*264);
	for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);	
	
	FlashChipSelect(FlashChip,HIGH);
	
	//  Wait until erase finishes
	while(!(FlashStatus(1)&0x80))  ;
	
}

//  Transfer contents of buffer 1 or 2 to Flash Page
//  Wait until done
void FlashPageWrite(int FlashChip,int page,int bufferno)
{
	int i;
	
	FlashChipSelect(FlashChip,LOW);
	
	//  Erase a Flash page
	if(bufferno==1)  SPI_MasterTransmit(0x88);
	else SPI_MasterTransmit(0x88);
	 
	// Convert address to 24 bit address
	ConvertFlashAddress(page*264);
	for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);	
	
	FlashChipSelect(FlashChip,HIGH);
	
	//  Wait until erase finishes
	while(!(FlashStatus(1)&0x80))  ;
}

//  Write bytes to Flash buffer
//    Note this routine does not check for wrap around - calling routine responsibility
//    This routine loads 16 bytes at a time
void FlashBytesToBuffer(int bufferno)
{
     int i;

     // Flash CS low	 
	 FlashChipSelect(1,LOW);
	 
	 //  Buffer write
	 if(bufferno==1) SPI_MasterTransmit(0x84);
	 else SPI_MasterTransmit(0x87);
	 
	 // Convert address to 24 bit address
	 ConvertFlashAddress(inteladdress);
	 FlashAddress[0] = 0x00;
	 for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
	 
	 for(i=0; i<bytesonline; i++)  SPI_MasterTransmit(intelbinary[i]);
	 
	 // Flash CS high
	 FlashChipSelect(1,HIGH);
}

// Write a single byte to a Flash Page - compound operation
void WriteFlashByte(int FlashChip,unsigned int address,unsigned char inbyte)
{
	int page;
	
	page = address/264;
	
	// Load Flash Page to buffer 1
	FlashPageToBuffer(FlashChip,page,1);
	
	// Erase page
	FlashPageErase(FlashChip,page);
	
	// Write byte to buffer 1
	inteladdress = address%264;
	intelbinary[0] = inbyte;
	bytesonline = 1;
	FlashBytesToBuffer(1);
	
	// Program Flash Page
	FlashPageWrite(FlashChip,page,1);
}

// Write Flash by passing data through buffers
void ProgramFlashPage(int FlashChip,unsigned int address,int length,int bufferno)
{
	 int i;

     // Flash CS low	 
	 FlashChipSelect(1,LOW);
	 
	 //  Buffer write then page program
	 if(bufferno==1) SPI_MasterTransmit(0x82);
	 else SPI_MasterTransmit(0x85);
	 
	 // Convert address to 24 bit address
	 ConvertFlashAddress(inteladdress);
	 FlashAddress[0] = 0x00;
	 for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
	 
	 for(i=0; i<bytesonline; i++)  SPI_MasterTransmit(intelbinary[i]);
	 
	 // Flash CS high
	 FlashChipSelect(1,HIGH);
}


//  Reset Flash Variables for next Intel Hex download
void   ResetFlashVars()
{
	currentflashbyteoffset = 0;  // Current byte in flash RAM buffer
	currentflashpage = 0;   // Current flash page being loaded
	currentflashchip = 1;   // Current flashchip being accessed - 1 or 2
	flashbufferloaded = NO;  // Status of flash buffer load - YES/NO
	flashpageprogrammed = NO; // Status of flash page programming - YES/NO
	flashbufferempty = YES;
	extendedaddress = 0;
	endofintelhex = YES;
}

//  Flash programming routine 
//     
//  When loading Intel Hex via the serial line this routine loads each line into the
//  Flash's RAM buffer so main memory is not used. As soon as sufficient data to fill 
//  one page is received the page is written.  The page is also written if the next
//  line contains data for a new page (or flash chip).
//
//  The Flash device contains two SRAM buffers of 264 or 528 bytes and
//  the following flash pages:
//      AT45DB041 - 2048 pages -  540672 bytes (264 byte)
//      AT45DB081 - 4096 pages - 1081344 bytes (264 byte)
//      AT45DB161 - 8192 pages - 2162688 bytes (528 byte)
//  Each Flash page takes 8 msec to erase and 12 msec to program or
//  20 msec to erase/program.  Each page is a certain length - the programming
//  process must wait for each page to be programmed. Note if Intel Hex is loaded
//  using fast serial then XON/XOFF flow control must be enabled in the serial
//  driver or an overrun condition will occur.
void ProgramFlash()
{
	unsigned char outbyte;
	int i,chipselect,bytesinpage,fpagesize,fbytesize,loopcnt=0,addressinpage;
	unsigned int address,bytesleftinpage;
	long superaddress,endaddress,storageendaddress;

	// Program flash page
	if(endofintelhex==YES) {
		if(flashpageprogrammed==NO) {
	 		FlashPageWrite(currentflashchip,currentflashpage,1);
	 		Delay(10000);
	 		flashpageprogrammed = YES;
	 		puts("Last page programmed\n\r"); 
			}
		ResetFlashVars();
		}
	// Load data and program flash page if RAM buffer full
	else {
		//  Convert Intel Address (with extended address) to RAM buffer address
		superaddress = extendedaddress*65536 + inteladdress;
		endaddress = superaddress + bytesonline - 1;
	
	puts("\n\r");
	puts("superaddress="); printlonghex(superaddress); puts("\n\r");
	puts("endaddress="); printlonghex(endaddress); puts("\n\r");
	
		// Calculate which Flash Chip to program 
		//   Take care of traverses from one Flash chip to the next
		if(superaddress<f1bytesize) {
			currentflashchip = 1;
			fpagesize = f1pagesize;
			fbytesize = f1bytesize;
			if(endaddress > (f1bytesize+f2bytesize))  endaddress = f1bytesize+f2bytesize;
			}
		// Start programming second Flash chip
		else if((superaddress>f1bytesize)&&(superaddress<(f1bytesize+f2bytesize))) {
			currentflashchip = 2;
			fpagesize = f2pagesize;
			fbytesize = f2bytesize;
			superaddress -= f1bytesize;
			endaddress -= f1bytesize;
			if(endaddress > (f1bytesize+f2bytesize)) endaddress = f2bytesize;
			}
		else currentflashchip = 3;  // Start range outside of Flash Chips

		if(currentflashchip<3) {
			currentflashpage = superaddress/fpagesize;
			addressinpage = superaddress%fpagesize;
			
	puts("addressinpage="); printint(addressinpage,5,0); puts("\n\r");

			if(bytesonline<(fpagesize-addressinpage)) bytesinpage = bytesonline;
			else bytesinpage = fpagesize-addressinpage;
			
	puts("currenflashpage="); printint(currentflashpage,5,0); puts("\n\r");
	puts("bytesinpage 2="); printint(bytesinpage,5,0); puts("\n\r");
	puts("bytesonline="); printint(bytesonline,5,0); puts("\n\r");
	
//		bytesonline -= bytesinpage;
			
	puts("bytesonline 2="); printint(bytesonline,5,0); puts("\n\r");
		
			loopcnt = 0;
			bytesleftinpage = 0;
	    	do  {

	    		// Load flash page if not loaded and erase page
	    		if(flashbufferloaded==NO) {
	    			FlashPageToBuffer(currentflashchip,currentflashpage,1);
	    			FlashPageErase(currentflashchip,currentflashpage);
	    			Delay(10000);  // delay 10msec
	    			flashbufferloaded = YES;
	    			flashpageprogrammed = NO;
	    			}

     			// Flash CS low	 
	 			FlashChipSelect(currentflashchip,LOW);
	 
	 			//  Put data in Flash RAM Buffer 1
	 			SPI_MasterTransmit(0x84);
	 
	 			// Convert address to 24 bit address
	 			ConvertFlashAddress(addressinpage);
	 		
//	 			puts("\n\r");
//	 			puts("Flash byte address=");
//	 			bytetohex(FlashAddress[0]); puts(hexbyte);
//	 			bytetohex(FlashAddress[1]); puts(hexbyte);
//	 			bytetohex(FlashAddress[2]); puts(hexbyte);
//	 			puts("\n\r");
//	 			puts("Current flash page="); printint(currentflashpage,5,0); puts("\n\r");
//				puts("bytesinpage="); printint(bytesinpage,5,0); puts("\n\r");
//				puts("bytesonline="); printint(bytesonline,5,0); puts("\n\r");
	 		
	 			for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
	 
	 			for(i=0; i<bytesinpage; i++)  SPI_MasterTransmit(intelbinary[i+bytesleftinpage]);
	 
	 			// Deselect flash chip
	 			FlashChipSelect(currentflashchip,HIGH);
	 		
	 			//  Write Buffer to Flash page if bytesonline exceeded page boundary
	 			if((addressinpage+bytesonline)>fpagesize)  {
	 				FlashPageWrite(currentflashchip,currentflashpage,1);
	 				Delay(10000);
	 				flashpageprogrammed = YES;
	 				flashbufferloaded = NO;
	 				currentflashpage++;
	 				addressinpage = 0;
	 				}
	 				
	 			//  Calculate next index parameters
	 			superaddress += bytesinpage;
				puts("superaddress="); printlonghex(superaddress); puts("\n\r");
			
	 			//  Check for Chip traverse - reset address if going to next chip
	 			//   TODO - doesn't handle traverse to next chip
				
	 			bytesleftinpage = bytesinpage;
				bytesinpage = bytesonline - bytesinpage;
				bytesonline = 0;		
	 		
	        	} while((superaddress<=endaddress)&&(loopcnt++<40));  // loopcnt is escape
			}
		}	
}


void Init_Flash()
{
	currentflashbyteoffset = 0;
	currentflashpage = 0;
	currentflashchip = 1;
	flashbufferloaded = NO;
	flashpageprogrammed = NO;
	
}

//  Fill Flash Memory with a byte value or from SDRAM
//   If length is longer than available storage program upto available storage
//   Other routine checks Flash configuration
void FillFlashMemory(long startaddress,long length,unsigned char databyte,unsigned char fillmode)
{
	unsigned char outbyte;
	int i,chipselect,bytesinpage,fpagesize,fbytesize,loopcnt,addressinpage;
	long superaddress,endaddress,storageendaddress,address;
	
	address = startaddress;
	endaddress = startaddress + length;
	
	// Calculate which Flash Chip to program 
	//   Take care of traverses from one Flash chip to the next
	if(address<f1bytesize) {
		chipselect = 1;
		fpagesize = f1pagesize;
		fbytesize = f1bytesize;
		if(endaddress > (f1bytesize+f2bytesize))  endaddress = f1bytesize+f2bytesize;
		}
	// Start programming second Flash chip
	else if((address>f1bytesize)&&(address<(f1bytesize+f2bytesize))) {
		chipselect = 2;
		fpagesize = f2pagesize;
		fbytesize = f2bytesize;
		startaddress -= f1bytesize;
		endaddress -= f1bytesize;
		if(endaddress > (f1bytesize+f2bytesize)) endaddress = f2bytesize;
		}
	else chipselect = 3;  // Start range outside of Flash Chips

	if(chipselect<3) {
		currentflashpage = startaddress/fpagesize;
		addressinpage = startaddress%fpagesize;
//	puts("addressinpage="); printint(addressinpage,5,0); puts("\n\r");
		if((addressinpage==0)&&(length>=fpagesize))  bytesinpage = fpagesize;
		else if(length<(fpagesize-addressinpage)) bytesinpage = length;
		else bytesinpage = fpagesize-addressinpage;
//	puts("currenflashpage="); printint(currentflashpage,5,0); puts("\n\r");
//	puts("bytesinpage 2="); printint(bytesinpage,5,0); puts("\n\r");
//	puts("length="); printlonghex(length); puts("\n\r");
		length -= bytesinpage;
//	puts("length 2="); printlonghex(length); puts("\n\r");
		
		loopcnt = 0;
	    do  {

	    	if(bytesinpage<fpagesize) FlashPageToBuffer(chipselect,currentflashpage,1);
	    	FlashPageErase(chipselect,currentflashpage);
	    	Delay(10000);
	    	
	    	// Fill buffer with byte
     		// Flash CS low	 
	 		FlashChipSelect(chipselect,LOW);
	 
	 		//  Buffer 1 write
	 		SPI_MasterTransmit(0x84);
	 
	 		// Convert address to 24 bit address
	 		//  TODO - convert this to superaddress 
	 		extendedaddress = superaddress/65536;
	 		address = superaddress - extendedaddress*65536;
	 		ConvertFlashAddress(address);
	 		
	 		for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
	 
	 		for(i=0; i<bytesinpage; i++)  {
	 			if(fillmode==FROMSDRAM)  SPI_MasterTransmit(*(pSDRAMBot+fillindex++));
	 			else SPI_MasterTransmit(databyte);
	 			}
	 
	 		// Flash CS high
	 		FlashChipSelect(chipselect,HIGH);
	 		
	 		//  Write Buffer to Flash page
	 		FlashPageWrite(chipselect,currentflashpage,1);
	 		Delay(10000);
	 		
	 		//  Calculate next index parameters
	 		superaddress += bytesinpage;
//			puts("superaddress="); printlonghex(superaddress); puts("\n\r");
			
	 		//  Check for Chip traverse - reset address if going to next chip
	 		if((chipselect==1)&&(superaddress>=f1bytesize)) superaddress = 0;
			currentflashpage = superaddress/fpagesize;
			addressinpage = superaddress%fpagesize;
			if((addressinpage==0) && (length>=fpagesize))  bytesinpage = fpagesize;
			else if(length<(fpagesize-addressinpage)) bytesinpage = length;
			else bytesinpage = fpagesize-addressinpage;
			length -= bytesinpage;	 		
	 		
	        } while(address<=endaddress);
		    
		}
}




void CheckFlash()
{
	unsigned char inbyte;
	
	inbyte = FlashStatus(1);
	if((inbyte&FLASHIDMASK)==AT45DB041ID) {
		f1pagesize = 264;
		f1bytesize = 540672;
		}
	else if((inbyte&FLASHIDMASK)==AT45DB081ID) {
		f1pagesize = 264;
		f1bytesize = 540672*2;
		}	
	else if((inbyte&FLASHIDMASK)==AT45DB161ID) {
		f1pagesize = 528;
		f1bytesize = 540672*4;
		}
	else {
		f1pagesize = 0;
		f1bytesize = 0;
		}
	inbyte = FlashStatus(2);
	if((inbyte&FLASHIDMASK)==AT45DB041ID) {
		f2pagesize = 264;
		f2bytesize = 540672;
		}
	else if((inbyte&FLASHIDMASK)==AT45DB081ID) {
		f2pagesize = 264;
		f2bytesize = 540672*2;
		}	
	else if((inbyte&FLASHIDMASK)==AT45DB161ID) {
		f2pagesize = 528;
		f2bytesize = 540672*4;
		}
	else {
		f2pagesize = 0;
		f2bytesize = 0;
		}		
}


