//*****************************************************************
//
//  LDRReader  - Parse a Blackfin LDR binary image and print record
//               headers
//
//  Version: 1.01    Date:  Dec 7, 2004
//  
//  A Blackfin LDR iamge consists of one or more blocks.  Each block
//  has a 10 header following by data.  The 10 byte header
//  contains a 32 bit start address, 32 bit byte count and 16 bit flag word.
//  The Start address defines the location in memory where the block is loaded.
//  The Count is the number of bytes in the data section of the block.
//
//  The program reads blocks until bit 15 of the Flag word is set indicating 
//  this is the last block.
//  
//  Flag Structure:
//    Bit 0 - ZEROFILL 0->non-zero fill block, 1->zero fill block
//    Bit 1 - RESVECT - 0->ADSP-BF532, 1->ADSP-BF533
//    Bit 3 - INIT - 0->Non-Init block, 1->Init block
//    Bit 4 - IGNORE - 0->Non-ignore block, 1->ignore block
//    Bit 8-5 - PFLAG 3:0 - PF number for SPI slave booting
//    Bit 15 - FINAL 0->non-last block, 1->Last block
//
//  Copyright 2004, SOC Machines, Inc.  All rights reserved.
//
//*****************************************************************
#include "P1_Main.h"
#include "ccblkfn.h"
#include "sysreg.h" 

#define ZEROFILLBIT	0x0001
#define RESVECTBIT	0x0002
#define INITBIT		0x0008
#define IGNOREBIT	0x0010
#define PFLAGBITS	0x01E0
#define FINALBIT	0x8000

extern char hexbyte[3],hexaddress[5];
extern unsigned int bytesonline,inteladdress,extendedaddress;
unsigned long addressptr;

unsigned char ReadSDRAMByte(unsigned long address)
{
	return *(pSDRAMBot+address);
}

void BinaryFlag(unsigned short flagword)
{
	short i;

	PrintHexWord(flagword); 
	puts("  ");
	for(i=0; i<16; i++)  {
		if(i==8) putchar(' ');
		if(flagword&0x8000) putchar('1');
			else putchar('0');
		flagword <<= 1;
		}
}

unsigned long getlong()
{
	unsigned char inbyte;
	unsigned long word;

	inbyte = ReadSDRAMByte(addressptr++);
	word = inbyte + (ReadSDRAMByte(addressptr++)*256);
	word += (ReadSDRAMByte(addressptr++)*256*256);
	word += (ReadSDRAMByte(addressptr++)*256*256*256);
	return word;
}

unsigned int getword()
{
	unsigned char inbyte;
	unsigned short word;

	inbyte = ReadSDRAMByte(addressptr++);
	word = inbyte + (ReadSDRAMByte(addressptr++)*256);
	return word;
}

void LoadElfImage(unsigned long bootaddress)
{
	unsigned char crcvalue,intelbinary[16],inchar;
	char instrg[10],outstrg[10],infilename[80],outfilename[80];
	unsigned int  flagword;
	int  i,j,bytesonline,notfinished,bytecnt,blockcnt;
	unsigned long longaddress,longbytecount;

	longaddress = 0;
	blockcnt = 0;
	notfinished = 1;
	addressptr = bootaddress;

	//  Start output Hex data
	puts("\n\r");
	while(notfinished) {
		// Read Loader block header
		longaddress = getlong();
		longbytecount = getlong();
		flagword = getword();
		blockcnt++;

		// Output header data
		puts("Block "); printint(blockcnt,4,0); puts(" header information\n\r");
		puts("  Start address (hex,decimal): ");
		printlonghex(longaddress);
		puts("\n\r  Byte count (hex,decimal): ");
		printlonghex(longbytecount);
		puts("\n\r  Flag word: "); BinaryFlag(flagword);
		puts("\n\r    Zero fill[");  
		    if(flagword&ZEROFILLBIT) puts("1]");
			else puts("0]");
		puts("   Resvect[");
			if(flagword&RESVECTBIT)  puts("1]");
			else puts("0]");
		puts("  Init[");
			if(flagword&INITBIT)  puts("1]\n\r");
			else puts("0]\n\r");
		puts("    Ignore[");
			if(flagword&IGNOREBIT)  puts("1]");
			else puts("0]");
		puts("    PFlag PF"); 
			inchar = ((flagword&PFLAGBITS)>>5)&0x000F;
			PrintHexByte(inchar);
		puts("    Final bit ");
			if(flagword&FINALBIT)  puts("Set\n\r");
			else puts("Not Set\n\r");
		
		if(flagword&FINALBIT) notfinished = 0;  // Last block if Final Bit set

		
		// Read block data
		else {
			for(i=0; i<longbytecount; i++)  {
				*(pSDRAMBot+longaddress+i) = ReadSDRAMByte(addressptr++);
				inchar = ReadSDRAMByte(addressptr++);
				// Escape - hit any key
				if(getcharnw()) {
					notfinished = 0;
					break;
					}
				}

			}
		}
}

