// *******************************************************
//
//   Program:  I2C.c        Version: 1.1 
//
//   Originally written By P. Fletcher-Jones on 16/11/01
//   
//   Modified to work on the P1 Blackfin processor
//   Modifications Copyright 2004, SOC Machines, Inc. 
//
//   The I2C functionality is used to control two PF lines
//   of the Blackfin PF port.  These lines are used to 
//   communicate via video input adapters connected to the 
//   P1 video port (connector J3, J17).
//
//   PF assignments are as follows:
//
//      PF0 - SDA   - data line
//      PF1 - SCL   - clock line
//
//   SDA (PF0) is a bi-directional data line.
//
// *******************************************************

// ****************************** //
// *** I2C Hardware Interface *** //
// ****************************** //

//  Need to assign SDA (PF0) and SCL (PF1) to SDA and SCL
//  SDA is PF0, SCL is PF1
#define  SDA  0x0003
#define  SCL  0x0003

#define I2C_100KHZ  4
#define I2C_400KHZ  1
#define I2C_PERIOD  2

//  SAA7113 Slave address if RTS0 high immediately after Reset release
//#define   SAA7113_Write_ADD  0x25
//#define   SAA7113_Read_ADD   0x26

#define   SAA7113_Write_ADD  0x4A
#define   SAA7113_Read_ADD   0x4B

//#define   SAA7113_Write_ADD  0x48
//#define   SAA7113_Read_ADD   0x49

#define   WRITECOMMAND  1
#define   READCOMMAND   2

#define   SKIPEVEN		1
#define	  SKIPODD		2
#define	  NOSKIP		3

//  Function definitions
void Set_SDA(char);
void Set_SCL(char);
void Init_I2C(void);
void Pulse_SCL(void);
void I2C_Start (void);
void I2C_Stop (void);
unsigned char I2C_Ackn(void);
int Write_I2C_Control(unsigned char);
int Write_I2C_Byte(unsigned char);
unsigned char Read_I2C_Byte(void);
int Write_CM64(unsigned char, unsigned char);
unsigned char Read_CM64(unsigned char);
int Write_CM200(unsigned char, unsigned char);
unsigned char Read_CM200(unsigned char);
void Reset_SAA7113(void);
void Reset_CM200(void);
void Init_SAA7113(short);
void Init_CM64(short,unsigned char);
void Halt_VideoDMAPPI(void);
void Init_ICM109S(void);


