// **************************************************************************
//
//  File:  P1_ISRs.c				Version: 0.51
//
//  Date:  October 20, 2004		
//
//  Interrupt service routines for P1 Blackfin
//
//  Copyright 2004, SOC Machines, Inc.
//
// **************************************************************************
//#include <cdefBF532.h>
#include <sys\exception.h>		//Interrupt Handling Header
#include "P1_Main.h"

/* Static Variables */
extern unsigned char UART_RxBuf[UART_RX_BUFFER_SIZE];
extern unsigned char UART_RxHead;
extern unsigned char UART_RxTail;
extern unsigned char UART_TxBuf[UART_TX_BUFFER_SIZE];
extern unsigned char UART_TxHead;
extern unsigned char UART_TxTail;
extern char  xoffsent, xonsent, enablexonoff, echoinchars, sendxonxoff;
extern char  rxcnt, checkflag;

//--------------------------------------------------------------------------//
// Function:	PPI_DMA_CH0													//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This ISR is executed every time Timer0 expires.				//
//				The old LED pattern is shifted by one; the direction		//
//				depends on the state of sLight_Move_Direction, which is		//
//				changed in FlagA_ISR.										//
//--------------------------------------------------------------------------//
EX_INTERRUPT_HANDLER(DMA0_PPI_ISR)	// <--|declaration in  exception.h -->
{									//    |declaration with _pragma(interrupt) the ISR Startaddress 


	//disable the interrupt request
	*pDMA0_IRQ_STATUS = 0x1;	
	
	puts( "\nThe DMA0 PPI Interrupt has been entered!\n" );

}//end DMA0_PPI_ISR


//--------------------------------------------------------------------------//
// Function:	Timer0_ISR													//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This ISR is executed every time Timer0 expires.				//
//				The old LED pattern is shifted by one; the direction		//
//				depends on the state of sLight_Move_Direction, which is		//
//				changed in FlagA_ISR.										//
//--------------------------------------------------------------------------//
EX_INTERRUPT_HANDLER(Timer0_ISR)
{
	static unsigned char ucActive_LED = 0x01;

	// confirm interrupt handling
	*pTIMER_STATUS = 0x0001;

}

//--------------------------------------------------------------------------//
// Function:	UART_TX_ISR													//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This ISR is executed every time the UART TX buffer becomes	//
//				empty.  The ISR supports a circular TX buffer with xon/xoff	//
//				flow control. If no characters to transmit turn Ints off	//
//--------------------------------------------------------------------------//
EX_INTERRUPT_HANDLER(UART_TX_ISR)
{
	unsigned char tmptail;

	// Check if all data is transmitted
	if(UART_TxHead != UART_TxTail)  {
	    // check chars in rx buffer - if too many trans XOFF
	    if(sendxonxoff == YES)  {
			 *pUART_THR = XOFF;
//			 UDR = '%';
			 xoffsent = YES;
			 sendxonxoff = NO;
			 }
		else {
			 // Calculate buffer index			 
			 tmptail = UART_TxTail + 1;
			 if(tmptail>=UART_TX_BUFFER_SIZE) tmptail = 0;
			 UART_TxTail = tmptail; 	// store new index
			 *pUART_THR = UART_TxBuf[tmptail]; // start transmission
			 if(UART_TxBuf[tmptail]==XON) {
			 	  xoffsent = NO;
				  xonsent = NO;
				  }
			 }
		}
	// check chars in rx buffer - if too many trans XOFF
	else  {  
		if(sendxonxoff == YES)  {
			 *pUART_THR = XOFF;
//			 UDR = '$';
			 xoffsent = YES;
			 sendxonxoff = NO;
			 }
		// disable ETBEI interrupt
		else {
			*pUART_IER &= 0xFFFD;
		//	*pSIC_IMASK = 0x0000C000;
			}
		}
}

//--------------------------------------------------------------------------//
// Function:	UART_RX_ISR													//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This ISR is executed every time a character is received into//
//				the rx buffer. The ISR supports a circular RX buffer with   //
//				xon/xoff flow control.												//
//--------------------------------------------------------------------------//
EX_INTERRUPT_HANDLER(UART_RX_ISR)
{
	unsigned char data;
	unsigned char tmphead;
	
	// Read the received data
	data = *pUART_RBR;
	 
	// Calculate buffer index
	tmphead = (UART_RxHead + 1);
	if(tmphead>=UART_RX_BUFFER_SIZE) tmphead = 0;
	UART_RxHead = tmphead; 		// store new index
	UART_RxBuf[tmphead] = data; // store received data in buffer
	
	rxcnt++;
	if(rxcnt>0x3f) checkflag = YES;
	if((rxcnt>(UART_RX_BUFFER_SIZE-20))&&(xoffsent==NO)&&(enablexonoff==YES)) {
		// If tx buffer is empty then transmit xoff else signal tx to send xoff
	    sendxonxoff = YES;
	    xoffsent = YES;
		//  Fire TX interrupt
		*pUART_IER |= ETBEI;		
		}
	// Does the RX buffer flag need clearing?
	
//	if ( tmphead == UART_RxTail )
//		{
//		/* ERROR! Receive buffer overflow */
//		}
}


